@@ -27,6 +27,8 @@ For a file starting with the bytes:
 
 The values for width and height would be 200, 400
 """
+from __future__ import annotations
+from typing import Optional, Union
 import struct
 
 
@@ -34,7 +36,8 @@ class IO:
 	"""
 	Class to handle i/o to a byte buffer or file-like object
 	"""
-	def __init__(self, data=None, idx=0, littleEndian=False, boolSize=8, stringEncoding='U'):
+	def __init__(self, data: Optional[bytearray]=None, idx: int=0,
+	littleEndian: bool=False, boolSize: int=8, stringEncoding: str='U'):
 		"""
 		:param data: can be a data buffer or a file-like object
 		:param idx: start reading/writing the data at the given index
@@ -45,25 +48,41 @@ class IO:
 		if data is None:
 			data = bytearray()
 		self._data = data
-		self.index = idx
+		self._index = idx
 		self._contexts = []
 		self.littleEndian = littleEndian
 		self.boolSize = boolSize
 		self.stringEncoding = stringEncoding # A=Ascii, U=UTF-8, W-Unicode wide
 
+	def __len__(self):
+		return len(self.data)
+
+	def __getitem__(self, idx: int):
+		return self.data[idx]
+
 	@property
-	def data(self):
+	def data(self) -> bytearray:
 		""" return data """
 		return self._data
 
 	@data.setter
-	def data(self, data):
+	def data(self, data: bytearray) -> None:
 		""" set data """
 		if not hasattr(data, "__getitem__"):
 			raise Exception('ERR: incorrect type for data buffer' + str(type(data)))
 		self._data = data
 
-	def beginContext(self, newIndex):
+	@property
+	def index(self) -> int:
+		""" return data """
+		return self._index
+
+	@index.setter
+	def index(self, index: int) -> None:
+		""" set index """
+		self._index = index
+
+	def beginContext(self, newIndex: int):
 		"""
 		Start a new context where the index can be changed all you want,
 		and when endContext() is called, it will be restored to the current position
@@ -77,30 +96,30 @@ class IO:
 		"""
 		self.index = self._contexts.pop()
 
-	def _write(self, size, fmt, data):
+	def _write(self, size: int, fmt: str, data: Union[str, bytearray, IO, int, float]):
 		"""	general formatted write	"""
 		if self.index + size >= len(self.data):
 			self.data.extend(bytearray((self.index + size) - len(self.data)))
 		try:
 			struct.pack_into(fmt, self.data, self.index, data)
 			self.index += size
-		except (DeprecationWarning, struct.error):
-			raise Exception(type(data), fmt, size, data)
+		except (DeprecationWarning, struct.error) as upstream:
+			raise Exception(type(data), fmt, size, data) from upstream
 
-	def _read(self, size, fmt):
+	def _read(self, size: int, fmt: str) -> Union[str, bytearray, IO, int, float]:
 		"""	general formatted read """
 		try:
 			data = struct.unpack(fmt, self.data[self.index:self.index + size])[0]
-		except (DeprecationWarning, struct.error):
+		except (DeprecationWarning, struct.error) as upstream:
 			raise Exception(str(fmt) + " " + str(size) + " " +
 			str(self.index) + ' ' + str(len(self.data)) + ' ' +
-			str(self.data[self.index:self.index + size]))
+			str(self.data[self.index:self.index + size])) from upstream
 		# Move the pointer forward
 		self.index += size
 		return data
 
 	@property
-	def bool(self):
+	def boolean(self):
 		""" return bool """
 		if self.boolSize == 8:
 			return self.bool8
@@ -112,8 +131,8 @@ class IO:
 			return self.bool64
 		raise Exception("Unknown bool size " + str(self.boolSize))
 
-	@bool.setter
-	def bool(self, ioBool):
+	@boolean.setter
+	def boolean(self, ioBool: Union[str, bytearray, IO, int, float]):
 		""" set bool """
 		if self.boolSize == 8:
 			self.bool8 = ioBool
@@ -127,134 +146,134 @@ class IO:
 			raise Exception("Unknown bool size " + str(self.boolSize))
 
 	@property
-	def bool8(self):
+	def bool8(self) -> Union[str, bytearray, IO, int, float]:
 		""" get bool8 """
 		return self.u8 != 0
 
 	@bool8.setter
-	def bool8(self, ioBool):
+	def bool8(self, ioBool: Union[str, bytearray, IO, int, float]):
 		""" set a bool8 """
 		self.u8 = ioBool
 
 	@property
-	def bool16(self):
+	def bool16(self) -> Union[str, bytearray, IO, int, float]:
 		""" get bool16 """
 		return self.u16 != 0
 
 	@bool16.setter
-	def bool16(self, ioBool):
+	def bool16(self, ioBool: Union[str, bytearray, IO, int, float]):
 		""" set bool16 """
 		self.u16 = ioBool
 
 	@property
-	def bool32(self):
+	def bool32(self) -> Union[str, bytearray, IO, int, float]:
 		""" get bool32 """
 		return self.u32 != 0
 
 	@bool32.setter
-	def bool32(self, ioBool):
+	def bool32(self, ioBool: Union[str, bytearray, IO, int, float]):
 		""" set bool32 """
 		self.u32 = ioBool
 
 	@property
-	def bool64(self):
+	def bool64(self) -> Union[str, bytearray, IO, int, float]:
 		""" get bool64 """
 		return self.u64 != 0
 
 	@bool64.setter
-	def bool64(self, ioBool):
+	def bool64(self, ioBool: Union[str, bytearray, IO, int, float]):
 		""" set bool64 """
 		self.u64 = ioBool
 
 	@property
-	def byte(self):
+	def byte(self) -> Union[str, bytearray, IO, int, float]:
 		""" get byte """
 		return self.i8
 
 	@byte.setter
-	def byte(self, byte):
+	def byte(self, byte: Union[str, bytearray, IO, int, float]):
 		""" set byte """
 		self.i8 = byte
 
 	@property
-	def unsignedByte(self):
+	def unsignedByte(self) -> Union[str, bytearray, IO, int, float]:
 		""" get unsigned byte """
 		return self.u8
 
 	@unsignedByte.setter
-	def unsignedByte(self, byte):
+	def unsignedByte(self, byte: Union[str, bytearray, IO, int, float]):
 		""" set unsigned byte """
 		self.u8 = byte
 
 	@property
-	def word(self):
+	def word(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a word """
 		return self.i16
 
 	@word.setter
-	def word(self, word):
+	def word(self, word: Union[str, bytearray, IO, int, float]):
 		""" set a word """
 		self.i16 = word
 
 	@property
-	def unsignedWord(self):
+	def unsignedWord(self) -> Union[str, bytearray, IO, int, float]:
 		""" get an unsigned word """
 		return self.u16
 
 	@unsignedWord.setter
-	def unsignedWord(self, unsignedWord):
+	def unsignedWord(self, unsignedWord: Union[str, bytearray, IO, int, float]):
 		""" set an unsigned word """
 		self.u16 = unsignedWord
 
 	@property
-	def dword(self):
+	def dword(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a dword """
 		return self.i32
 
 	@dword.setter
-	def dword(self, dword):
+	def dword(self, dword: Union[str, bytearray, IO, int, float]):
 		""" set a dword """
 		self.i32 = dword
 
 	@property
-	def unsignedDword(self):
+	def unsignedDword(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a unsigned dword """
 		return self.u32
 
 	@unsignedDword.setter
-	def unsignedDword(self, unsignedDword):
+	def unsignedDword(self, unsignedDword: Union[str, bytearray, IO, int, float]):
 		""" set an unsigned dword """
 		self.u32 = unsignedDword
 
 	@property
-	def qword(self):
+	def qword(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a qword """
 		return self.i64
 
 	@qword.setter
-	def qword(self, qword):
+	def qword(self, qword: Union[str, bytearray, IO, int, float]):
 		""" set a qword """
 		self.i64 = qword
 
 	@property
-	def unsignedQword(self):
+	def unsignedQword(self) -> Union[str, bytearray, IO, int, float]:
 		""" get an unsigned qword """
 		return self.u64
 
 	@unsignedQword.setter
-	def unsignedQword(self, unsignedQword):
+	def unsignedQword(self, unsignedQword: Union[str, bytearray, IO, int, float]):
 		""" set an unsigned qword """
 		self.u64 = unsignedQword
 
 	@property
-	def i8(self):
+	def i8(self) -> Union[str, bytearray, IO, int, float]:
 		""" get an int8 """
 		if self.littleEndian:
 			return self.i8le
 		return self.i8be
 
 	@i8.setter
-	def i8(self, i8):
+	def i8(self, i8: Union[str, bytearray, IO, int, float]):
 		""" set an int8 """
 		if self.littleEndian:
 			self.i8le = i8
@@ -262,14 +281,14 @@ class IO:
 			self.i8be = i8
 
 	@property
-	def u8(self):
+	def u8(self) -> Union[str, bytearray, IO, int, float]:
 		""" get an unsigned int """
 		if self.littleEndian:
 			return self.u8le
 		return self.u8be
 
 	@u8.setter
-	def u8(self, u8):
+	def u8(self, u8: Union[str, bytearray, IO, int, float]):
 		""" set an unsigned int """
 		if self.littleEndian:
 			self.u8le = u8
@@ -277,14 +296,14 @@ class IO:
 			self.u8be = u8
 
 	@property
-	def i16(self):
+	def i16(self) -> Union[str, bytearray, IO, int, float]:
 		""" get an int16 """
 		if self.littleEndian:
 			return self.i16le
 		return self.i16be
 
 	@i16.setter
-	def i16(self, i16):
+	def i16(self, i16: Union[str, bytearray, IO, int, float]):
 		""" set an int16 """
 		if self.littleEndian:
 			self.i16le = i16
@@ -292,14 +311,14 @@ class IO:
 			self.i16be = i16
 
 	@property
-	def u16(self):
+	def u16(self) -> Union[str, bytearray, IO, int, float]:
 		""" get an uint16 """
 		if self.littleEndian:
 			return self.u16le
 		return self.u16be
 
 	@u16.setter
-	def u16(self, u16):
+	def u16(self, u16: Union[str, bytearray, IO, int, float]):
 		""" set an unint16 """
 		if self.littleEndian:
 			self.u16le = u16
@@ -307,14 +326,14 @@ class IO:
 			self.u16be = u16
 
 	@property
-	def i32(self):
+	def i32(self) -> Union[str, bytearray, IO, int, float]:
 		""" get an int32 """
 		if self.littleEndian:
 			return self.i32le
 		return self.i32be
 
 	@i32.setter
-	def i32(self, i32):
+	def i32(self, i32: Union[str, bytearray, IO, int, float]):
 		"""set an int32 """
 		if self.littleEndian:
 			self.i32le = i32
@@ -322,14 +341,14 @@ class IO:
 			self.i32be = i32
 
 	@property
-	def u32(self):
+	def u32(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a uint32 """
 		if self.littleEndian:
 			return self.u32le
 		return self.u32be
 
 	@u32.setter
-	def u32(self, u32):
+	def u32(self, u32: Union[str, bytearray, IO, int, float]):
 		""" set a unint32 """
 		if self.littleEndian:
 			self.u32le = u32
@@ -337,14 +356,14 @@ class IO:
 			self.u32be = u32
 
 	@property
-	def i64(self):
+	def i64(self) -> Union[str, bytearray, IO, int, float]:
 		""" get an int64 """
 		if self.littleEndian:
 			return self.i64le
 		return self.i64be
 
 	@i64.setter
-	def i64(self, i64):
+	def i64(self, i64: Union[str, bytearray, IO, int, float]):
 		""" set an int64 """
 		if self.littleEndian:
 			self.i64le = i64
@@ -352,14 +371,14 @@ class IO:
 			self.i64be = i64
 
 	@property
-	def u64(self):
+	def u64(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a uint64 """
 		if self.littleEndian:
 			return self.u64le
 		return self.u64be
 
 	@u64.setter
-	def u64(self, u64):
+	def u64(self, u64: Union[str, bytearray, IO, int, float]):
 		""" set a uint64 """
 		if self.littleEndian:
 			self.u64le = u64
@@ -367,14 +386,14 @@ class IO:
 			self.u64be = u64
 
 	@property
-	def float32(self):
+	def float32(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a float32 """
 		if self.littleEndian:
 			return self.float32le
 		return self.float32be
 
 	@float32.setter
-	def float32(self, float32):
+	def float32(self, float32: Union[str, bytearray, IO, int, float]):
 		""" set a float32 """
 		if self.littleEndian:
 			self.float32le = float32
@@ -382,14 +401,14 @@ class IO:
 			self.float32be = float32
 
 	@property
-	def float64(self):
+	def float64(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a float64 """
 		if self.littleEndian:
 			return self.float64le
 		return self.float64be
 
 	@float64.setter
-	def float64(self, float64):
+	def float64(self, float64: Union[str, bytearray, IO, int, float]):
 		""" set a float64 """
 		if self.littleEndian:
 			self.float64le = float64
@@ -397,234 +416,234 @@ class IO:
 			self.float64be = float64
 
 	@property
-	def u8be(self):
+	def u8be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next uint8 and advance the index """
 		return self._read(1, ">B")
 
 	@u8be.setter
-	def u8be(self, u8be):
+	def u8be(self, u8be: Union[str, bytearray, IO, int, float]):
 		""" set the uint8 """
 		self._write(1, '>B', u8be)
 
 	@property
-	def u8le(self):
+	def u8le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next uint8 and advance the index """
 		return self._read(1, "<B")
 
 	@u8le.setter
-	def u8le(self, u8le):
+	def u8le(self, u8le: Union[str, bytearray, IO, int, float]):
 		""" set the uint8 """
 		self._write(1, '<B', u8le)
 
 	@property
-	def i8le(self):
+	def i8le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next signed int8 and advance the index """
 		return self._read(1, "<b")
 
 	@i8le.setter
-	def i8le(self, i8le):
+	def i8le(self, i8le: Union[str, bytearray, IO, int, float]):
 		""" set the int8 """
 		self._write(1, '<b', i8le)
 
 	@property
-	def i8be(self):
+	def i8be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next signed int8 and advance the index """
 		return self._read(1, ">b")
 
 	@i8be.setter
-	def i8be(self, i8be):
+	def i8be(self, i8be: Union[str, bytearray, IO, int, float]):
 		""" set the int8 """
 		self._write(1, '>b', i8be)
 
 	@property
-	def u16be(self):
+	def u16be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next uint16 and advance the index """
 		return self._read(2, ">H")
 
 	@u16be.setter
-	def u16be(self, u16be):
+	def u16be(self, u16be: Union[str, bytearray, IO, int, float]):
 		""" set the uint16 """
 		self._write(2, '>H', u16be)
 
 	@property
-	def u16le(self):
+	def u16le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next uint16 and advance the index """
 		return self._read(2, "<H")
 
 	@u16le.setter
-	def u16le(self, u16le):
+	def u16le(self, u16le: Union[str, bytearray, IO, int, float]):
 		""" set the uint16 """
 		self._write(2, '<H', u16le)
 
 	@property
-	def i16le(self):
+	def i16le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next signed int16 and advance the index """
 		return self._read(2, "<h")
 
 	@i16le.setter
-	def i16le(self, i16le):
+	def i16le(self, i16le: Union[str, bytearray, IO, int, float]):
 		""" set the int16 """
 		self._write(2, '<h', i16le)
 
 	@property
-	def i16be(self):
+	def i16be(self) -> Union[str, bytearray, IO, int, float]:
 		""" read the next signed int16 and advance the index """
 		return self._read(2, ">h")
 
 	@i16be.setter
-	def i16be(self, i16be):
+	def i16be(self, i16be: Union[str, bytearray, IO, int, float]):
 		""" set the int16 """
 		self._write(2, '>h', i16be)
 
 	@property
-	def u32be(self):
+	def u32be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next uint32 and advance the index """
 		return self._read(4, ">I")
 
 	@u32be.setter
-	def u32be(self, u32be):
+	def u32be(self, u32be: Union[str, bytearray, IO, int, float]):
 		""" set the uint32 """
-		self._write(4, '>I', int(u32be))
+		self._write(4, '>I', u32be)
 
 	@property
-	def u32le(self):
+	def u32le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next uint32 and advance the index """
 		return self._read(4, "<I")
 
 	@u32le.setter
-	def u32le(self, u32le):
+	def u32le(self, u32le: Union[str, bytearray, IO, int, float]):
 		""" set the uint32 """
 		self._write(4, '<I', u32le)
 
 	@property
-	def i32le(self):
+	def i32le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next signed int32 and advance the index """
 		return self._read(4, "<i")
 
 	@i32le.setter
-	def i32le(self, i32le):
+	def i32le(self, i32le: Union[str, bytearray, IO, int, float]):
 		""" set the int32 """
 		self._write(4, '<i', i32le)
 
 	@property
-	def i32be(self):
+	def i32be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next signed int32 and advance the index """
 		return self._read(4, ">i")
 
 	@i32be.setter
-	def i32be(self, i32be):
+	def i32be(self, i32be: Union[str, bytearray, IO, int, float]):
 		""" set the int32 """
-		self._write(4, '>i', int(i32be))
+		self._write(4, '>i', i32be)
 
 	@property
-	def u64be(self):
+	def u64be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next uint64 and advance the index """
 		return self._read(8, ">Q")
 
 	@u64be.setter
-	def u64be(self, u64be):
+	def u64be(self, u64be: Union[str, bytearray, IO, int, float]):
 		""" set the uint64 """
 		self._write(8, '>Q', u64be)
 
 	@property
-	def u64le(self):
+	def u64le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next uint64 and advance the index """
 		return self._read(8, "<Q")
 
 	@u64le.setter
-	def u64le(self, u64le):
+	def u64le(self, u64le: Union[str, bytearray, IO, int, float]):
 		""" set the uint64 """
 		self._write(8, '<Q', u64le)
 
 	@property
-	def i64le(self):
+	def i64le(self) -> Union[str, bytearray, IO, int, float]:
 		""" read the next signed int64 and advance the index """
 		return self._read(8, "<q")
 
 	@i64le.setter
-	def i64le(self, i64le):
+	def i64le(self, i64le: Union[str, bytearray, IO, int, float]):
 		""" set the int64 """
 		self._write(8, '<q', i64le)
 
 	@property
-	def i64be(self):
+	def i64be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next signed int64 and advance the index """
 		return self._read(8, ">q")
 
 	@i64be.setter
-	def i64be(self, i64be):
+	def i64be(self, i64be: Union[str, bytearray, IO, int, float]):
 		""" set the int64 """
 		self._write(8, '>q', i64be)
 
 	@property
-	def float(self):
+	def floating(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a float """
 		return self.float32
 
-	@float.setter
-	def float(self, f):
+	@floating.setter
+	def floating(self, floating: Union[str, bytearray, IO, int, float]):
 		""" set a float """
-		self.float32 = f
+		self.float32 = floating
 
 	@property
-	def double(self):
+	def double(self) -> Union[str, bytearray, IO, int, float]:
 		""" get a double """
 		return self.float64
 
 	@double.setter
-	def double(self, f):
+	def double(self, floating: Union[str, bytearray, IO, int, float]):
 		""" set a double """
-		self.float64 = f
+		self.float64 = floating
 
 	@property
-	def float32be(self):
+	def float32be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next 32 bit float and advance the index """
 		return self._read(4, ">f")
 
 	@float32be.setter
-	def float32be(self, float32be):
+	def float32be(self, float32be: Union[str, bytearray, IO, int, float]):
 		""" set a 32 bit float """
 		self._write(4, '>f', float32be)
 
 	@property
-	def float32le(self):
+	def float32le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next 32 bit float and advance the index """
 		return self._read(4, "<f")
 
 	@float32le.setter
-	def float32le(self, float32le):
+	def float32le(self, float32le: Union[str, bytearray, IO, int, float]):
 		""" set a 32 bit float """
 		self._write(4, '<f', float32le)
 
 	@property
-	def float64be(self):
+	def float64be(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next 64 bit float and advance the index """
 		return self._read(8, ">d")
 
 	@float64be.setter
-	def float64be(self, float64be):
+	def float64be(self, float64be: Union[str, bytearray, IO, int, float]):
 		""" set a 64 bit float """
 		self._write(8, '>d', float64be)
 
 	@property
-	def float64le(self):
+	def float64le(self) -> Union[str, bytearray, IO, int, float]:
 		"""	read the next 64 bit float and advance the index """
 		return self._read(8, "<d")
 
 	@float64le.setter
-	def float64le(self, float64le):
+	def float64le(self, float64le: Union[str, bytearray, IO, int, float]):
 		""" set a 64 bit float """
 		self._write(8, '<d', float64le)
 
-	def getBytes(self, nbytes):
+	def getBytes(self, nbytes: int):
 		"""
 		grab some raw bytes and advance the index
 		"""
-		d = self.data[self.index:self.index + nbytes]
+		data = self.data[self.index:self.index + nbytes]
 		self.index += nbytes
-		return d
+		return data
 
-	def addBytes(self, ioBytes):
+	def addBytes(self, ioBytes: Union[str, bytearray, IO, int, float]):
 		"""
 		add some raw bytes and advance the index
 
@@ -634,7 +653,7 @@ class IO:
 		"""
 		self.setBytes(ioBytes)
 
-	def setBytes(self, ioBytes):
+	def setBytes(self, ioBytes: Union[str, bytearray, IO, int, float]):
 		"""
 		add some raw bytes and advance the index
 
@@ -653,11 +672,11 @@ class IO:
 		else:
 			if self.index + len(ioBytes) >= len(self.data):
 				self.data.extend(bytearray((self.index + len(ioBytes)) - len(self.data)))
-			for b in ioBytes:
-				self.data[self.index] = b
+			for ioByte in ioBytes:
+				self.data[self.index] = ioByte
 				self.index += 1
 
-	def _sz754(self, encoding):
+	def _sz754(self, encoding: str):
 		"""
 		Read the next string conforming to IEEE 754 and advance the index
 
@@ -670,70 +689,70 @@ class IO:
 		nchars = self.u32
 		if nchars == 0:
 			return ''
-		d = self.data[self.index:self.index + nchars - 1]
+		data = self.data[self.index:self.index + nchars - 1]
 		self.index += nchars
 		if encoding == 'A':
-			return d
+			return data
 		if encoding == 'U':
-			return d.decode('UTF-8', errors='replace')
+			return data.decode('UTF-8', errors='replace')
 		if encoding == 'W':
-			return d.decode('UTF-16', errors='replace')
+			return data.decode('UTF-16', errors='replace')
 		raise Exception()
 
-	def _sz754set(self, sz754, _encoding):
+	def _sz754set(self, sz754: Union[str, bytearray, IO, int, float], _encoding: str):
 		'''_sz754set'''
 		self.u32 = len(sz754)
 		self.setBytes(sz754)
 		self.u8 = 0
 
 	@property
-	def sz754(self):
+	def sz754(self) -> Union[str, bytearray, IO, int, float]:
 		'''sz754'''
 		return self._sz754(self.stringEncoding)
 
 	@sz754.setter
-	def sz754(self, sz754):
+	def sz754(self, sz754: Union[str, bytearray, IO, int, float]):
 		'''set sz754'''
 		return self._sz754set(sz754, self.stringEncoding)
 
 	@property
-	def sz754A(self):
+	def sz754A(self) -> Union[str, bytearray, IO, int, float]:
 		'''sz754A'''
 		return self._sz754('A')
 
 	@sz754A.setter
-	def sz754A(self, sz754):
+	def sz754A(self, sz754: Union[str, bytearray, IO, int, float]):
 		'''set sz754A'''
 		return self._sz754set(sz754, 'A')
 
 	@property
-	def sz754W(self):
+	def sz754W(self) -> Union[str, bytearray, IO, int, float]:
 		'''sz754W'''
 		return self._sz754('W')
 
 	@sz754W.setter
-	def sz754W(self, sz754):
+	def sz754W(self, sz754: Union[str, bytearray, IO, int, float]):
 		'''set sz754W'''
 		return self._sz754set(sz754, 'W')
 
 	@property
-	def sz754U(self):
+	def sz754U(self) -> Union[str, bytearray, IO, int, float]:
 		'''sz754U'''
 		return self._sz754('U')
 
 	@sz754U.setter
-	def sz754U(self, sz754):
+	def sz754U(self, sz754: Union[str, bytearray, IO, int, float]):
 		'''set sz754U'''
 		return self._sz754set(sz754, 'U')
 
-	def _readUntil(self, until, encoding='A'):
+	def _readUntil(self, until: Union[str, bytearray, IO, int, float], encoding: str='A') -> str:
 		"""
 		Read a sequence of chars in a set encoding until a set char
 
 		:param until: must be within the ascii character set
 		:param encoding: one of A (ascii), U (UTF-8) or W (UCS-2)
 		"""
-		d = []
+		data = []
 		if encoding == 'A':
 			encoding = 'ascii'
 		elif encoding == 'U':
@@ -744,19 +763,19 @@ class IO:
 			raise Exception('bogus encoding')
 		until = until.encode('ascii')[0] # always ascii
 		while True:
-			c = self.data[self.index]
+			char = self.data[self.index]
 			self.index += 1
 			if encoding == 'W': # get one more byte
-				d.append(c)
-				c = self.data[self.index]
+				data.append(char)
+				char = self.data[self.index]
 				self.index += 1
-			if c == until:
+			if char == until:
 				break
-			d.append(c)
-		return bytes(d).decode(encoding, errors='replace')
+			data.append(char)
+		return bytes(data).decode(encoding, errors='replace')
 
 	@property
-	def textLine(self):
+	def textLine(self) -> Union[str, bytearray, IO, int, float]:
 		'''Read a sequence of chars until the next new line char'''
 		ret = self._readUntil('\n', self.stringEncoding)
 		if ret[-1] == '\r':
@@ -764,14 +783,14 @@ class IO:
 		return ret
 
 	@textLine.setter
-	def textLine(self, text):
+	def textLine(self, text: Union[str, bytearray, IO, int, float]):
 		'''Set a sequence of chars until the next new line char'''
 		self.setBytes(text)
-		if text[-1] != '\n':
+		if isinstance(text, (int, float)) or text[-1] != '\n':
 			self.setBytes('\n')
 
 	@property
-	def textLineA(self):
+	def textLineA(self) -> Union[str, bytearray, IO, int, float]:
 		'''Read a sequence of chars until the next new line char in ascii'''
 		ret = self._readUntil('\n', 'A')
 		if ret[-1] == '\r':
@@ -779,14 +798,14 @@ class IO:
 		return ret
 
 	@textLineA.setter
-	def textLineA(self, text):
+	def textLineA(self, text: Union[str, bytearray, IO, int, float]):
 		'''Set a sequence of chars until the next new line char in ascii'''
 		self.setBytes(text)
-		if text[-1] != '\n':
+		if isinstance(text, (int, float)) or text[-1] != '\n':
 			self.setBytes('\n')
 
 	@property
-	def textLineW(self):
+	def textLineW(self) -> Union[str, bytearray, IO, int, float]:
 		'''Read a sequence of chars until the next new line char in ucs-2'''
 		ret = self._readUntil('\n', 'W')
 		if ret[-1] == '\r':
@@ -794,14 +813,14 @@ class IO:
 		return ret
 
 	@textLineW.setter
-	def textLineW(self, text):
+	def textLineW(self, text: Union[str, bytearray, IO, int, float]):
 		'''Set a sequence of chars until the next new line char in ucs-2'''
 		self.setBytes(text)
-		if text[-1] != '\n':
+		if isinstance(text, (int, float)) or text[-1] != '\n':
 			self.setBytes('\0\n')
 
 	@property
-	def textLineU(self):
+	def textLineU(self) -> Union[str, bytearray, IO, int, float]:
 		'''Read a sequence of chars until the next new line char in utf-8'''
 		ret = self._readUntil('\n', 'U')
 		if ret[-1] == '\r':
@@ -809,52 +828,52 @@ class IO:
 		return ret
 
 	@textLineU.setter
-	def textLineU(self, text):
+	def textLineU(self, text: Union[str, bytearray, IO, int, float]):
 		'''Set a sequence of chars until the next new line char in utf-8'''
 		self.setBytes(text)
-		if text[-1] != '\n':
+		if isinstance(text, (int, float)) or text[-1] != '\n':
 			self.setBytes('\n')
 
 	@property
-	def cString(self):
+	def cString(self) -> Union[str, bytearray, IO, int, float]:
 		'''Read a sequence of chars until the next null byte'''
 		return self._readUntil('\0', self.stringEncoding)
 
 	@cString.setter
-	def cString(self, text):
+	def cString(self, text: Union[str, bytearray, IO, int, float]):
 		"""Set a sequence of chars and add a null byte """
 		self.setBytes(text)
 		self.setBytes('\0')
 
 	@property
-	def cStringA(self):
+	def cStringA(self) -> Union[str, bytearray, IO, int, float]:
 		'''Read a sequence of chars until the next null byte in ascii'''
 		return self._readUntil('\0', 'A')
 
 	@cStringA.setter
-	def cStringA(self, text):
+	def cStringA(self, text: Union[str, bytearray, IO, int, float]):
 		"""Set a sequence of chars and add a null byte in ascii"""
 		self.setBytes(text)
 		self.setBytes('\0')
 
 	@property
-	def cStringW(self):
+	def cStringW(self) -> Union[str, bytearray, IO, int, float]:
 		'''Read a sequence of chars until the next null byte in ucs-2'''
 		return self._readUntil('\0', 'W')
 
 	@cStringW.setter
-	def cStringW(self, text):
+	def cStringW(self, text: Union[str, bytearray, IO, int, float]):
 		"""Set a sequence of chars and add a null byte in ucs-2"""
 		self.setBytes(text)
 		self.setBytes('\0\0')
 
 	@property
-	def cStringU(self):
+	def cStringU(self) -> Union[str, bytearray, IO, int, float]:
 		'''Read a sequence of chars until the next null byte in utf-8'''
 		return self._readUntil('\0', 'U')
 
 	@cStringU.setter
-	def cStringU(self, text):
+	def cStringU(self, text: Union[str, bytearray, IO, int, float]):
 		"""Set a sequence of chars and add a null byte in utf-8"""
 		self.setBytes(text)
 		self.setBytes('\0')
